--------------------------------------------------------------------------------
-- Copyright (c) 1995-2003 Xilinx, Inc.
-- All Right Reserved.
--------------------------------------------------------------------------------
--   ____  ____ 
--  /   /\/   / 
-- /___/  \  /    Vendor: Xilinx 
-- \   \   \/     Version : 8.2.03i
--  \   \         Application : ISE
--  /   /         Filename : teszt.vhw
-- /___/   /\     Timestamp : Thu Nov 16 15:08:47 2006
-- \   \  /  \ 
--  \___\/\___\ 
--
--Command: 
--Design Name: teszt
--Device: Xilinx
--

library UNISIM;
use UNISIM.Vcomponents.ALL;
library ieee;
use ieee.std_logic_1164.ALL;
use ieee.numeric_std.ALL;
USE IEEE.STD_LOGIC_TEXTIO.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
USE STD.TEXTIO.ALL;

ENTITY teszt IS
END teszt;

ARCHITECTURE testbench_arch OF teszt IS
    FILE RESULTS: TEXT OPEN WRITE_MODE IS "results.txt";

    COMPONENT blokksema
        PORT (
            clk50 : In std_logic;
            nyomogomb : In std_logic;
            rst : In std_logic;
            digit : Out std_logic_vector (3 DownTo 0);
            led : Out std_logic;
            seg : Out std_logic_vector (7 DownTo 0)
        );
    END COMPONENT;

    SIGNAL clk50 : std_logic := '0';
    SIGNAL nyomogomb : std_logic := '0';
    SIGNAL rst : std_logic := '1';
    SIGNAL digit : std_logic_vector (3 DownTo 0) := "0000";
    SIGNAL led : std_logic := '0';
    SIGNAL seg : std_logic_vector (7 DownTo 0) := "00000000";

    SHARED VARIABLE TX_ERROR : INTEGER := 0;
    SHARED VARIABLE TX_OUT : LINE;

    constant PERIOD : time := 200 ns;
    constant DUTY_CYCLE : real := 0.5;
    constant OFFSET : time := 0 ns;

    BEGIN
        UUT : blokksema
        PORT MAP (
            clk50 => clk50,
            nyomogomb => nyomogomb,
            rst => rst,
            digit => digit,
            led => led,
            seg => seg
        );

        PROCESS    -- clock process for clk50
        BEGIN
            WAIT for OFFSET;
            CLOCK_LOOP : LOOP
                clk50 <= '0';
                WAIT FOR (PERIOD - (PERIOD * DUTY_CYCLE));
                clk50 <= '1';
                WAIT FOR (PERIOD * DUTY_CYCLE);
            END LOOP CLOCK_LOOP;
        END PROCESS;

        PROCESS
            PROCEDURE CHECK_digit(
                next_digit : std_logic_vector (3 DownTo 0);
                TX_TIME : INTEGER
            ) IS
                VARIABLE TX_STR : String(1 to 4096);
                VARIABLE TX_LOC : LINE;
                BEGIN
                IF (digit /= next_digit) THEN
                    STD.TEXTIO.write(TX_LOC, string'("Error at time="));
                    STD.TEXTIO.write(TX_LOC, TX_TIME);
                    STD.TEXTIO.write(TX_LOC, string'("ns digit="));
                    IEEE.STD_LOGIC_TEXTIO.write(TX_LOC, digit);
                    STD.TEXTIO.write(TX_LOC, string'(", Expected = "));
                    IEEE.STD_LOGIC_TEXTIO.write(TX_LOC, next_digit);
                    STD.TEXTIO.write(TX_LOC, string'(" "));
                    TX_STR(TX_LOC.all'range) := TX_LOC.all;
                    STD.TEXTIO.writeline(RESULTS, TX_LOC);
                    STD.TEXTIO.Deallocate(TX_LOC);
                    ASSERT (FALSE) REPORT TX_STR SEVERITY ERROR;
                    TX_ERROR := TX_ERROR + 1;
                END IF;
            END;
            PROCEDURE CHECK_led(
                next_led : std_logic;
                TX_TIME : INTEGER
            ) IS
                VARIABLE TX_STR : String(1 to 4096);
                VARIABLE TX_LOC : LINE;
                BEGIN
                IF (led /= next_led) THEN
                    STD.TEXTIO.write(TX_LOC, string'("Error at time="));
                    STD.TEXTIO.write(TX_LOC, TX_TIME);
                    STD.TEXTIO.write(TX_LOC, string'("ns led="));
                    IEEE.STD_LOGIC_TEXTIO.write(TX_LOC, led);
                    STD.TEXTIO.write(TX_LOC, string'(", Expected = "));
                    IEEE.STD_LOGIC_TEXTIO.write(TX_LOC, next_led);
                    STD.TEXTIO.write(TX_LOC, string'(" "));
                    TX_STR(TX_LOC.all'range) := TX_LOC.all;
                    STD.TEXTIO.writeline(RESULTS, TX_LOC);
                    STD.TEXTIO.Deallocate(TX_LOC);
                    ASSERT (FALSE) REPORT TX_STR SEVERITY ERROR;
                    TX_ERROR := TX_ERROR + 1;
                END IF;
            END;
            PROCEDURE CHECK_seg(
                next_seg : std_logic_vector (7 DownTo 0);
                TX_TIME : INTEGER
            ) IS
                VARIABLE TX_STR : String(1 to 4096);
                VARIABLE TX_LOC : LINE;
                BEGIN
                IF (seg /= next_seg) THEN
                    STD.TEXTIO.write(TX_LOC, string'("Error at time="));
                    STD.TEXTIO.write(TX_LOC, TX_TIME);
                    STD.TEXTIO.write(TX_LOC, string'("ns seg="));
                    IEEE.STD_LOGIC_TEXTIO.write(TX_LOC, seg);
                    STD.TEXTIO.write(TX_LOC, string'(", Expected = "));
                    IEEE.STD_LOGIC_TEXTIO.write(TX_LOC, next_seg);
                    STD.TEXTIO.write(TX_LOC, string'(" "));
                    TX_STR(TX_LOC.all'range) := TX_LOC.all;
                    STD.TEXTIO.writeline(RESULTS, TX_LOC);
                    STD.TEXTIO.Deallocate(TX_LOC);
                    ASSERT (FALSE) REPORT TX_STR SEVERITY ERROR;
                    TX_ERROR := TX_ERROR + 1;
                END IF;
            END;
            BEGIN
                -- -------------  Current Time:  285ns
                WAIT FOR 285 ns;
                rst <= '0';
                -- -------------------------------------
                -- -------------  Current Time:  885ns
                WAIT FOR 600 ns;
                nyomogomb <= '1';
                -- -------------------------------------
                -- -------------  Current Time:  1485ns
                WAIT FOR 600 ns;
                nyomogomb <= '0';
                -- -------------------------------------
                -- -------------  Current Time:  2485ns
                WAIT FOR 1000 ns;
                nyomogomb <= '1';
                -- -------------------------------------
                -- -------------  Current Time:  3085ns
                WAIT FOR 600 ns;
                nyomogomb <= '0';
                -- -------------------------------------
                -- -------------  Current Time:  3685ns
                WAIT FOR 600 ns;
                nyomogomb <= '1';
                -- -------------------------------------
                -- -------------  Current Time:  4285ns
                WAIT FOR 600 ns;
                nyomogomb <= '0';
                -- -------------------------------------
                WAIT FOR 15915 ns;

                IF (TX_ERROR = 0) THEN
                    STD.TEXTIO.write(TX_OUT, string'("No errors or warnings"));
                    STD.TEXTIO.writeline(RESULTS, TX_OUT);
                    ASSERT (FALSE) REPORT
                      "Simulation successful (not a failure).  No problems detected."
                      SEVERITY FAILURE;
                ELSE
                    STD.TEXTIO.write(TX_OUT, TX_ERROR);
                    STD.TEXTIO.write(TX_OUT,
                        string'(" errors found in simulation"));
                    STD.TEXTIO.writeline(RESULTS, TX_OUT);
                    ASSERT (FALSE) REPORT "Errors found during simulation"
                         SEVERITY FAILURE;
                END IF;
            END PROCESS;

    END testbench_arch;

